import datetime
import email
import errno
import fcntl
import logging
import logging.handlers
import os
import subprocess

from django.conf import settings

def get_harvest_version(version_file, debug):
    """
    return the bzr revision number and version of Harvest
    """

    if not os.path.exists(version_file):
        return "version unknown"

    f = email.message_from_file(open(version_file))
    version = f["version"]
    bzr_revno = f["revno"]
    
    if debug:
        try:            
            from bzrlib.branch import Branch
            branch = Branch.open_containing('.')[0]
            bzr_revno = branch.revno()
        except:
            pass

    return "version %s (r%s)" % (version, bzr_revno)


def get_harvest_version_name(version_file, debug):
    """
    return a short string describing the version of harvest
    """
    
    version_str = ""
    
    if debug:
        try:
            from bzrlib.branch import Branch
            branch = Branch.open_containing('.')[0]
            version_str = "Revision %s" % branch.revno()
        except:
            pass
    elif os.path.exists(version_file):
        f = email.message_from_file(open(version_file))
        if "versionname" in f:
            version_str = f["versionname"]
    
    return version_str

def instantiate_logger(job_name):
    logger = logging.getLogger(job_name)
    stamp_dir = os.path.join(settings.PROJECT_PATH, 'data')
    if not os.path.exists(stamp_dir):
        os.makedirs(stamp_dir)
    log_file = os.path.join(stamp_dir, "%s.log" % job_name)
    rfh = logging.handlers.RotatingFileHandler(log_file, maxBytes=100000, backupCount=5)
    logger.addHandler(rfh)
    logger.setLevel(settings.LOG_LEVEL)
    return logger

def lock_path(path):
    f = open(path, "wb")
    try:
        fcntl.flock(f, fcntl.LOCK_EX | fcntl.LOCK_NB)
        return f
    except IOError, e:
        if e.errno in (errno.EACCES, errno.EAGAIN):
            return None
        raise

def update_stamp(stamp_file):
    open(stamp_file, "a").close()
    os.utime(stamp_file, None)

def run_job(which, interval, args=None):
    logger = instantiate_logger(which)
    stamp_dir = os.path.join(settings.PROJECT_PATH, 'data')
    if not os.path.exists(stamp_dir):
        os.makedirs(stamp_dir)
    stamp_file = os.path.join(stamp_dir, "%s.stamp" % which)
    lock_file = os.path.join(stamp_dir, "%s.lock" % which)
    if not os.path.exists(stamp_file) or \
       datetime.datetime.utcfromtimestamp(os.path.getmtime(stamp_file)) + interval < datetime.datetime.now():
        f = lock_path(lock_file)
        if f:
            pwd = os.getcwd()
            os.chdir(settings.PROJECT_PATH)
            p = subprocess.Popen(["./manage.py", which], stdout=subprocess.PIPE, 
                                  stderr=subprocess.STDOUT, close_fds=True)
            log = p.stdout.read()
            p.communicate()
            os.chdir(pwd)
            update_stamp(stamp_file)
            if log.strip():
                logger.error("STDOUT + STDERR Output\n%s" % log)
        else:
            logger.error("Couldn't get lock for job '%s'." % which)
