from django.utils.translation import ugettext as _

from harvest.common.ordereddict import OrderedDict
from harvest.filters import filters, containers
import models

class PkgNameFilter(filters.EditFilter):
    def process_queryset(self, queryset):
        return queryset.filter(name__startswith = self.get_value())

class PkgSetFilter(filters.ChoiceFilter):
    def __init__(self, id_str, **kwargs):
        choices = OrderedDict()
        for s in models.PackageSet.objects.all(): #TODO: find a way to sort these
            choices[s.name] = s
        
        filters.ChoiceFilter.__init__(
            self,
            id_str,
            choices_dict = choices,
            **kwargs)
    
    def process_queryset(self, queryset):
        return queryset.filter(packagesets__in = self.get_selected_choices())



class OppFeaturedFilter(filters.Filter):
    def process_queryset(self, queryset):
        return queryset.filter(opportunitylist__featured = True)

class OppListFilter(filters.ChoiceFilter):
    def __init__(self, id_str, **kwargs):
        choices = OrderedDict()
        for l in models.OpportunityList.objects.all(): #TODO: find a way to sort these
            if l.active:
                choices[l.name] = l
        
        filters.ChoiceFilter.__init__(
            self,
            id_str,
            choices_dict = choices,
            **kwargs)
    
    def process_queryset(self, queryset):
        return queryset.filter(opportunitylist__in = self.get_selected_choices())

    def render_html_value_choice(self, item_id):
        toggle_params = self.serialize(self.get_value_with_selection(item_id))
        item_href = self.get_system().get_url_with_parameters(toggle_params)
        title_attribute = ''
        help_html = ''
        if hasattr(self.choices_dict[item_id], 'description') and \
            self.choices_dict[item_id].description:
            title_attribute = 'title="%s"' % self.choices_dict[item_id].description
        if hasattr(self.choices_dict[item_id], 'explanation') and \
            self.choices_dict[item_id].explanation:
            help_html = '<a class="help" href="%s" target="_blank">?</a>' % \
                            self.choices_dict[item_id].explanation
        return '<a class="item-toggle" href="%s" %s>%s</a> %s' % (item_href, title_attribute, item_id, help_html)

class OppStatusFilter(filters.ChoiceFilter):
    def __init__(self, id_str, **kwargs):
        choices = OrderedDict.fromkeys(['applied', 'irrelevant'])
        labels = {'applied' : _("Applied"),
                  'irrelevant' : _("Irrelevant")}
        
        filters.ChoiceFilter.__init__(
            self,
            id_str,
            choices_dict = choices,
            choices_labels_dict = labels,
            **kwargs)
    
    def process_queryset(self, queryset):
        selection = self.get_value()
        if not 'applied' in selection:
            queryset = queryset.exclude(applied = True)
        if not 'irrelevant' in selection:
            queryset = queryset.exclude(reviewed = True)
        return queryset

#we don't really need to create a special type here, but it may be handy
class HarvestFilters(containers.FilterSystem):
    def __init__(self):
        containers.FilterSystem.__init__(
            self,
            [
                filters.FilterGroup('pkg', [
                    PkgNameFilter('name', name=_('Named')),
                    PkgSetFilter('set', name=_('Limit to'))
                ], name=_('Packages') ),
                filters.FilterGroup('opp', [
                    OppFeaturedFilter('featured', name=_('Featured')),
                    OppListFilter('list', name=_('Limit to')),
                    OppStatusFilter('status', name=_('Status'))
                ], name=_('Opportunities') )
            ],
            default_parameters = { 'pkg' : ['set'],
                                'pkg.set' : ['ubuntu-desktop'],
                                'opp' : ['list', 'status'],
                                'opp.list' : ['bitesize'] }
        )

